from django.conf.urls import url

from .views import ImageView

urlpatterns = [
    url(r'thumbnail/(?P<pk>\d+)/(?P<width>\d+|auto)/(?P<height>\d+|auto)/(?P<format>source|jpg|png|webp)/(?P<crop>[^/]+)/(?P<quality>\d+|60)/', ImageView.as_view(), name='thumbnail'),
]



# The default quality settings passed to Sorl
LAZY_IMAGE_DEFAULT_QUALITY_OPTIONS = {
    'normal': 60,
    'webp': 80,
}
LAZY_IMAGE_DEFAULT_QUALITY = LAZY_IMAGE_DEFAULT_QUALITY_OPTIONS['normal']

# This is the namespace you gave the package in your urls.py
LAZY_IMAGE_URL_NAMESPACE = 'assets'

# This is the file model that Sorl is expecting to create a thumbnail from
LAZY_IMAGE_FILE_MODEL = 'media.File'

# Do you want the lazy images to server webp where possible?
LAZY_IMAGE_ENABLE_WEBP = True



from django.apps import apps
from django.shortcuts import get_object_or_404
from django.views.generic import RedirectView
from sorl.thumbnail import get_thumbnail

from .conf import settings


class ImageView(RedirectView):
    # If they change the source image, we don't want to be showing the old image.
    # Sorl uses memcached to retrieve images with the same args, so this should
    # be pretty quick.
    permanent = False

    def get_redirect_url(self, *args, **kwargs):
        # kwargs:
        # {'pk': '9', 'width': '285', 'height': '400', 'format': 'webp', 'crop': 'None'}
        file_model = apps.get_model(*settings.LAZY_IMAGE_FILE_MODEL.split('.', 1))
        file_ = get_object_or_404(file_model, pk=kwargs['pk'])

        sorl_args = [
            file_.file,
        ]
        sorl_kwargs = {}

        dimensions = ''
        width = kwargs['width']
        height = kwargs['height']

        if width == 'auto':
            dimensions = 'x{}'.format(height)
        elif height == 'auto':
            dimensions = width
        else:
            dimensions = '{}x{}'.format(width, height)

            if 'crop' not in kwargs:
                kwargs['crop'] = 'center'

        sorl_args.append(dimensions)

        if kwargs['crop'].lower() != 'none':
            sorl_kwargs['crop'] = kwargs['crop']

        if kwargs['format'] != 'source':
            sorl_kwargs['format'] = kwargs['format'].upper()

        sorl_kwargs['quality'] = int(kwargs['quality'])

        return get_thumbnail(
            *sorl_args,
            **sorl_kwargs
        ).url



import json
import os
import sys

from setuptools import find_packages, setup
from setuptools.command.install import install

with open('package.json') as f:
    VERSION = json.load(f)['version']

def readme():
    """print long description"""
    with open('README.md') as f:
        return f.read()

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-lazy-image',
    version=VERSION,
    description='An image utility that renders images lazily as well as intrinsically sizes them.',
    long_description=readme(),
    packages=find_packages(),
    install_requires=[
        'sorl-thumbnail',
        'Jinja2',
        'django>=1.11,<1.12',
        'django-jinja>=2.2',
    ],
    extras_require={
        'testing': [
            'coveralls',
            'pytest',
            'pytest-cov',
            'pytest-django',
            'pylint',
            'pylint-django',
            'pylint-mccabe',
            'isort',
        ],
    },
    include_package_data=True,
    url='https://github.com/dan-gamble/django-lazy-image/',
    author='Dan Gamble',
    author_email='dan@dangamble.co.uk',
    python_requires='>=3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3 :: Only",
    ],
    cmdclass={
        'verify': VerifyVersionCommand,
    },
)



# pylint: disable=too-complex, too-many-arguments, too-many-locals

from django.core.urlresolvers import reverse
from django_jinja import library
from sorl.thumbnail import get_thumbnail

from ..conf import settings


@library.global_function
def lazy_image(
        image, height=None, width=None, blur=True, max_width=1920, crop=None,
        quality=settings.LAZY_IMAGE_DEFAULT_QUALITY, webp=settings.LAZY_IMAGE_ENABLE_WEBP):
    user_sized = height and width
    reverse_url = '{}:thumbnail'.format(settings.LAZY_IMAGE_URL_NAMESPACE)

    if isinstance(quality, int):
        normal_quality = webp_quality = quality
    elif isinstance(quality, dict):
        try:
            normal_quality = quality['normal']
            webp_quality = quality['webp']
        except KeyError:
            normal_quality = settings.LAZY_IMAGE_DEFAULT_QUALITY_OPTIONS['normal']
            webp_quality = settings.LAZY_IMAGE_DEFAULT_QUALITY_OPTIONS['webp']

    if user_sized and not crop:
        crop = 'center'

    # Ideally we will use the images uploaded sizes to get our aspect ratio but in certain circumstances, like cards,
    # we will use our own provided ones
    if not height:
        height = image.height

    if not width:
        width = image.width

    aspect_ratio = height / width if user_sized else image.height / image.width

    if width > max_width:
        width = max_width

    if width > height:
        height = int(width * aspect_ratio)

    # The aspect ratio will be used to size the image with a padding-bottom based element
    aspect_ratio_percentage = '{}%'.format(aspect_ratio * 100)

    original_large_image_url = reverse(reverse_url, kwargs={
        'pk': image.pk,
        'width': width,
        'height': height,
        'format': 'source',
        'crop': crop,
        'quality': normal_quality,
    })

    original_large_image_url_2x = reverse(reverse_url, kwargs={
        'pk': image.pk,
        'width': width * 2,
        'height': height * 2,
        'format': 'source',
        'crop': crop,
        'quality': normal_quality,
    })

    webp_url = reverse(reverse_url, kwargs={
        'pk': image.pk,
        'width': width,
        'height': height,
        'format': 'webp',
        'crop': crop,
        'quality': webp_quality,
    }) if webp else None

    webp_url_2x = reverse(reverse_url, kwargs={
        'pk': image.pk,
        'width': width * 2,
        'height': height * 2,
        'format': 'webp',
        'crop': crop,
        'quality': webp_quality,
    }) if webp else None

    try:
        small_image_url = get_thumbnail(image.file, str(int(width / 20))).url
    except ValueError:
        # Guard against really tiny images, i.e. width / 20 results in 0.
        small_image_url = original_large_image_url

    return {
        'alt_text': image.alt_text or '',
        'aspect_ratio': aspect_ratio_percentage,
        'small_image_url': small_image_url,
        'original_large_image_url': original_large_image_url,
        'original_large_image_url_2x': original_large_image_url_2x,
        'webp': webp,
        'webp_url': webp_url,
        'webp_url_2x': webp_url_2x,
        'blur': False if str(image.file).endswith('.png') else blur
    }


@library.global_function
@library.render_with('django_lazy_image/lazy-image.html')
def render_lazy_image(image, height=None, width=None, blur=True, max_width=1920, crop=None,
                      quality=settings.LAZY_IMAGE_DEFAULT_QUALITY, webp=settings.LAZY_IMAGE_ENABLE_WEBP):
    """
    Usage: {{ render_lazy_image(path.to.image) }}
    """

    return lazy_image(image, height, width, blur, max_width, crop, quality, webp)
